@@ -10,11 +10,13 @@ from airflow_commons.sql_resources import (
 from airflow_commons.utils.bigquery_utils import (
     DEFAULT_TIMEOUT,
     DEFAULT_LOCATION,
+    DEFAULT_RETURN_TYPE,
     connect,
     get_time_partition_field,
     single_value_select,
     query,
     select,
+    query_information_schema,
     get_table_ref,
 )
 from airflow_commons.utils.file_utils import read_sql
@@ -22,6 +24,7 @@ from airflow_commons.utils.sql_utils import (
     get_deduplication_source_statement,
     get_merge_sql,
     get_delete_sql,
+    get_deduplication_additional_merge_conditions,
 )
 from airflow_commons.utils.time_utils import get_buffered_timestamp
 
@@ -30,7 +33,6 @@ WRITE_DISPOSITIONS = ["WRITE_APPEND", "WRITE_TRUNCATE", "WRITE_EMPTY"]
 DEFAULT_PRIMARY_KEYS = ["id"]
 ARCHIVE_DEFAULT_PRIMARY_KEYS = ["id", "last_updated_at", "processed_at"]
 DEFAULT_TIME_COLUMNS = ["last_updated_at", "processed_at"]
-DEFAULT_RETURN_TYPE = "dataframe"
 
 
 def deduplicate(
@@ -78,6 +80,7 @@ def deduplicate(
         client, source_dataset, source_table
     )
     partition_pruning_params = dict()
+    additional_merge_conditions = None
     partition_pruning_params["target_partition_field"] = get_time_partition_field(
         client, target_dataset, target_table
     )
@@ -109,6 +112,9 @@ def deduplicate(
                 )
             )
             return None
+        additional_merge_conditions = get_deduplication_additional_merge_conditions(
+            partition_pruning_params=partition_pruning_params
+        )
     source_statement = get_deduplication_source_statement(
         start_date=buffered_start_date,
         end_date=end_date,
@@ -122,8 +128,6 @@ def deduplicate(
         allow_partition_pruning=allow_partition_pruning,
         partition_pruning_params=partition_pruning_params,
     )
-    if not allow_partition_pruning:
-        partition_pruning_params.clear()
     sql = get_merge_sql(
         client=client,
         project_id=project_id,
@@ -132,7 +136,7 @@ def deduplicate(
         source_statement=source_statement,
         primary_keys=primary_keys,
         mode="UPSERT",
-        **partition_pruning_params
+        additional_merge_conditions=additional_merge_conditions,
     )
     return query(
         client=client,
@@ -151,6 +155,7 @@ def upsert(
     target_table: str,
     source_statement_params: dict = None,
     primary_keys=None,
+    additional_merge_conditions: str = None,
     timeout=DEFAULT_TIMEOUT,
     location=DEFAULT_LOCATION,
 ):
@@ -164,6 +169,7 @@ def upsert(
     :param target_table: targeted table id
     :param source_statement_params: parameters of source statement
     :param primary_keys: target table's primary key list
+    :param additional_merge_conditions: additional user specified merge conditions as string
     :param timeout: query timeout duration in seconds
     :param location: query location parameter
     :return:
@@ -184,6 +190,7 @@ def upsert(
         source_statement=source_statement,
         primary_keys=primary_keys,
         mode="UPSERT",
+        additional_merge_conditions=additional_merge_conditions,
     )
     return query(
         client=client,
@@ -202,6 +209,7 @@ def insert(
     target_table: str,
     source_statement_params: dict = None,
     primary_keys=None,
+    additional_merge_conditions: str = None,
     timeout=DEFAULT_TIMEOUT,
     location=DEFAULT_LOCATION,
 ):
@@ -214,6 +222,7 @@ def insert(
     :param target_dataset: targeted dataset id
     :param target_table: targeted table id
     :param source_statement_params: parameters of source statement
+    :param additional_merge_conditions: additional user specified merge conditions as string
     :param primary_keys: target table's primary key list
     :param timeout: query timeout duration in seconds
     :param location: query location parameter
@@ -235,6 +244,7 @@ def insert(
         source_statement=source_statement,
         primary_keys=primary_keys,
         mode="INSERT",
+        additional_merge_conditions=additional_merge_conditions,
     )
     return query(
         client=client,
@@ -470,3 +480,21 @@ def archive(
         timeout=timeout,
         location=location,
     )
+
+
+def list_tables(service_account_file: str, project_id: str, dataset_id: str):
+    """
+    Returns a list of all tables in a dataset
+
+    :param service_account_file: relative location of service account file
+    :param project_id: Bigquery project id
+    :param dataset_id: dataset id
+    :return: List
+    """
+    client = connect(service_account_file=service_account_file)
+    return query_information_schema(
+        client=client,
+        requested_column_name="table_name",
+        project_id=project_id,
+        dataset_id=dataset_id,
+    )
