@@ -29,9 +29,6 @@ DELETE_SQL_FILE = os.path.join(here, "bigquery/dml/delete.sql")
 GET_COLUMN_NAMES_SQL_FILE = os.path.join(
     here, "bigquery/information_schema/get_column_names.sql"
 )
-GET_OBJECT_FIELDS_SQL_FILE = os.path.join(
-    here, "bigquery/information_schema/get_object_fields.sql"
-)
 GET_TABLE_NAMES_SQL_FILE = os.path.join(
     here, "bigquery/information_schema/get_table_names.sql"
 )
